/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  existsAsHtmlElement,
  getElementFromFixture,
  getElementFromFixtureByType,
  getElementsFromFixture,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { CommonModule } from '@angular/common';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatTooltipModule } from '@angular/material/tooltip';
import { ActivatedRoute } from '@angular/router';
import { ExclamationIconComponent, ListComponent, ListItemComponent } from '@ods/system';
import {
  AdminOrganisationsEinheitResource,
  AdminOrganisationsEinheitSyncResult,
} from 'libs/admin/organisations-einheit-shared/src/lib/organisations-einheit.model';
import { createAdminOrganisationsEinheitResource } from 'libs/admin/organisations-einheit-shared/src/test/organisations-einheit';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, MockModule } from 'ng-mocks';
import { OrganisationsEinheitListComponent } from './organisations-einheit-list.component';

describe('OrganisationsEinheitListComponent', () => {
  let component: OrganisationsEinheitListComponent;
  let fixture: ComponentFixture<OrganisationsEinheitListComponent>;

  const listSelector: string = getDataTestIdOf('organisations-einheit-list');
  const listItemSelector: string = getDataTestIdOf('organisations-einheit-list-item');
  const organisationsEinheitNameSelector: string = getDataTestIdOf('organisations-einheit-name');
  const organisationsEinheitIdSelector: string = getDataTestIdOf('organisations-einheit-id');

  const organisationsEinheitResource: AdminOrganisationsEinheitResource = {
    ...createAdminOrganisationsEinheitResource(),
    syncResult: AdminOrganisationsEinheitSyncResult.NOT_FOUND_IN_PVOG,
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      providers: [
        {
          provide: ActivatedRoute,
          useValue: mock(ActivatedRoute),
        },
      ],
      imports: [
        CommonModule,
        OrganisationsEinheitListComponent,
        MockComponent(ListComponent),
        MockComponent(ListItemComponent),
        MockComponent(ExclamationIconComponent),
        MockModule(MatTooltipModule),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitListComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('input', () => {
    describe('organisationsEinheitResources', () => {
      beforeEach(() => {
        component.organisationsEinheitResources = [organisationsEinheitResource];
        fixture.detectChanges();
      });

      it('should set organisationsEinheitResource name', () => {
        const nameElement: HTMLElement = getElementFromFixture(fixture, organisationsEinheitNameSelector);

        expect(nameElement.textContent).toBe(organisationsEinheitResource.name);
      });

      it('should set organisationsEinheitResource organisationsEinheitId', () => {
        const idElement: HTMLElement = getElementFromFixture(fixture, organisationsEinheitIdSelector);

        expect(idElement.textContent).toBe(organisationsEinheitResource.organisationsEinheitId);
      });

      it('should set exclamation icon', () => {
        const iconElement: ExclamationIconComponent = getElementFromFixtureByType(fixture, ExclamationIconComponent);

        expect(iconElement).toBeTruthy();
      });
    });
  });

  describe('component', () => {
    describe('moveChildrenIntoParentLevel', () => {
      it('should move children one level up', () => {
        const childList: AdminOrganisationsEinheitResource[] = [
          createAdminOrganisationsEinheitResource(),
          createAdminOrganisationsEinheitResource(),
        ];
        const item1: AdminOrganisationsEinheitResource = createAdminOrganisationsEinheitResource();
        const item2: AdminOrganisationsEinheitResource = createAdminOrganisationsEinheitResource();
        item2['_embedded'] = { childList };
        component.organisationsEinheitResources = [item1, item2];

        expect(component.organisationsEinheitResources.length).toBe(4);
      });

      it('should set isChild property at moved children', () => {
        const childList: AdminOrganisationsEinheitResource[] = [
          createAdminOrganisationsEinheitResource(),
          createAdminOrganisationsEinheitResource(),
        ];
        const item1: AdminOrganisationsEinheitResource = createAdminOrganisationsEinheitResource();
        const item2: AdminOrganisationsEinheitResource = createAdminOrganisationsEinheitResource();
        item2['_embedded'] = { childList };
        component.organisationsEinheitResources = [item1, item2];

        expect(component.organisationsEinheitResources[2].isChild).toBeTruthy();
      });
    });
  });

  describe('template', () => {
    describe('organisationsEinheiten list', () => {
      describe('without organisationsEinheiten', () => {
        it('should not show list', () => {
          notExistsAsHtmlElement(fixture, listSelector);
        });
      });

      describe('with organisationsEinheiten', () => {
        const organisationsEinheiten: AdminOrganisationsEinheitResource[] = [
          createAdminOrganisationsEinheitResource(),
          createAdminOrganisationsEinheitResource(),
        ];

        beforeEach(() => {
          component.organisationsEinheitResources = organisationsEinheiten;
          fixture.detectChanges();
        });

        it('should show list', () => {
          existsAsHtmlElement(fixture, listSelector);
        });

        it('should show rows', () => {
          const rows = getElementsFromFixture(fixture, listItemSelector);

          expect(rows.length).toEqual(2);
        });
      });
    });
  });
});
