/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Mock, mock } from '@alfa-client/test-utils';
import { TestBed, fakeAsync, tick } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import KcAdminClient from '@keycloak/keycloak-admin-client';
import { TokenProvider } from '@keycloak/keycloak-admin-client/lib/client';
import GroupRepresentation from '@keycloak/keycloak-admin-client/lib/defs/groupRepresentation';
import MappingsRepresentation from '@keycloak/keycloak-admin-client/lib/defs/mappingsRepresentation';
import { OAuthService } from 'angular-oauth2-oidc';
import { createUser } from '../../test/user';
import { User } from './user.model';
import { UserRepository } from './user.repository.service';

describe('UserRepository', () => {
  const accessToken: string = faker.string.alphanumeric(40);

  let repository: UserRepository;

  let kcAdminClient: Mock<KcAdminClient>;
  let oAuthService: Mock<OAuthService>;

  beforeEach(() => {
    kcAdminClient = mock(KcAdminClient);
    oAuthService = mock(OAuthService);
    TestBed.configureTestingModule({
      providers: [
        { provide: OAuthService, useValue: oAuthService },
        { provide: KcAdminClient, useValue: kcAdminClient },
      ],
    });
    oAuthService.getAccessToken.mockReturnValue(accessToken);
    repository = TestBed.inject(UserRepository);
  });

  it('should be created', () => {
    expect(repository).toBeTruthy();
  });

  describe('registerTokenProvider', () => {
    it('should register token provider from oauth service', async () => {
      const tokenProvider: TokenProvider = kcAdminClient.registerTokenProvider.mock.calls[0][0];
      const token: string = await tokenProvider.getAccessToken();
      expect(token).toEqual(accessToken);
    });
  });

  describe('getUsers', () => {
    const userRep: User = createUser();
    const userRepArray: User[] = [userRep, userRep, userRep];
    const group: string = faker.word.sample();
    const role: string = faker.word.sample();
    const groupRep: GroupRepresentation[] = [{ name: group }];
    const roleRep: MappingsRepresentation = {
      clientMappings: { alfa: { mappings: [{ name: role }] } },
    };

    beforeEach(() => {
      jest.clearAllMocks();
      kcAdminClient.users = <any>{
        find: jest.fn().mockReturnValue(Promise.resolve(userRepArray)),
        listGroups: jest.fn().mockReturnValue(Promise.resolve(groupRep)),
        listRoleMappings: jest.fn().mockReturnValue(Promise.resolve(roleRep)),
      };
    });

    it('should call kcAdminClient users find', () => {
      repository.getUsers();

      expect(kcAdminClient.users['find']).toHaveBeenCalled();
    });

    it('should call mapToUser', fakeAsync(() => {
      const mapToUser: jest.SpyInstance<User> = jest.spyOn(repository, 'mapToUser');

      repository.getUsers().subscribe();
      tick();

      expect(mapToUser).toBeCalledTimes(userRepArray.length);
    }));

    it('should call kcadminClient listGroups for every user', fakeAsync(() => {
      repository.getUsers().subscribe();
      tick();

      expect(kcAdminClient.users['listGroups']).toBeCalledTimes(userRepArray.length);
    }));

    it('should call kcadminClient listRoleMappings for every user', fakeAsync(() => {
      repository.getUsers().subscribe();
      tick();

      expect(kcAdminClient.users['listRoleMappings']).toBeCalledTimes(userRepArray.length);
    }));

    it('should return users with groups and roles', (done) => {
      repository.getUsers().subscribe((users: User[]) => {
        users.forEach((user) => expect(user).toEqual({ ...userRep, groups: [group], roles: [role] }));
        done();
      });
    });

    it('should return users with empty groups and roles if they have none', (done) => {
      kcAdminClient.users['listGroups'] = jest.fn().mockReturnValue(Promise.resolve([]));
      kcAdminClient.users['listRoleMappings'] = jest.fn().mockReturnValue(Promise.resolve({}));

      repository.getUsers().subscribe((users: User[]) => {
        users.forEach((user) => expect(user).toEqual({ ...userRep, groups: [], roles: [] }));
        done();
      });
    });
  });
});
